#!/usr/bin/env python
# -*- coding: utf8 -*-

#-----------------------------------------
# This is a part of Collège 3D project.
# Author:       Pascal Peter
# Copyright:    (C) 2003-2016 Pascal Peter
# Licence:      GNU General Public Licence version 3
# Website:      http://pascal.peter.free.fr/
# Email:        pascal.peter at free.fr
#-----------------------------------------

"""
DESCRIPTION :
    Définition d'un client réseau gérant en parallèle l'émission
    et la réception des messages (utilisation de 2 THREADS).
    d'après Swinnen : http://inforef.be/swi/python.htm
"""

# importation des modules utiles :
import socket
import threading
from time import time

import bge



# **************************************************
#       PARAMÈTRES DE CONNEXION
# **************************************************
BUFFER = 1024



# **************************************************
#       THREAD DE RÉCEPTION
# **************************************************
class ThreadReception(threading.Thread):
    """
    objet thread gérant la réception des messages
    """
    def __init__(self, connexion):
        threading.Thread.__init__(self)
        # réf. du socket de connexion
        self.connexion = connexion
        self.received_message = ''

    def run(self):
        while True:
            try:
                received_message = self.connexion.recv(BUFFER).decode('Utf8')
                if 'SERVER' in received_message:
                    received_message = received_message.replace('øø', 'ø')
                if not('øø' in received_message):
                    bge.logic.globalDict['LAN']['received_message'] = received_message
                    self.received_message = received_message
            except socket.error:
                received_message = 'END'
            if not(received_message) or (received_message.upper() == 'END'):
                # Le thread <réception> se termine ici.
                break
        print('CLIENT DISCONNECTED')
        self.connexion.close()



# **************************************************
#       THREAD D'ÉMISSION
# **************************************************
class ThreadEmission(threading.Thread):
    """
    objet thread gérant l'émission des messages
    """
    def __init__(self, connexion):
        threading.Thread.__init__(self)
        self.connexion = connexion

    def run(self):
        if not('sent_message' in bge.logic.globalDict):
            bge.logic.globalDict['LAN']['sent_message'] = ''
        while True:
            try:
                sent_message = bge.logic.globalDict['LAN']['sent_message']
            except:
                sent_message = ''
                break
            if sent_message != '':
                self.connexion.send(sent_message.encode('Utf8'))
                bge.logic.globalDict['LAN']['sent_message'] = ''
                if sent_message.upper() == 'END':
                    # Le thread <émission> se termine ici.
                    break
        self.connexion.close()



# **************************************************
#       THREAD D'ÉMISSION SIMPLIFIÉ
#       (pour test de connexion)
# **************************************************
class ThreadEmissionTest(threading.Thread):
    """
    objet thread gérant l'émission des messages
    """
    def __init__(self, connexion):
        threading.Thread.__init__(self)
        self.connexion = connexion

    def run(self):
        self.connexion.send('TEST'.encode('Utf8'))

    def fin(self):
        self.connexion.send('END'.encode('Utf8'))



# **************************************************
#       POUR TESTER LA CONNEXION
# **************************************************
def testConnexion(host, port):
    connexion = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        connexion.connect((host, port))
    except socket.error:
        return False, []

    # Dialogue avec le serveur : on lance deux threads pour gérer
    # indépendamment l'émission et la réception des messages :
    th_E = ThreadEmissionTest(connexion)
    th_R = ThreadReception(connexion)
    th_E.start()
    th_R.start()
    testTime = time()
    while not('OTHERS' in th_R.received_message):
        waiting = True
        if time() - testTime > 5:
            th_E.fin()
            return False, []
    try:
        others = th_R.received_message.split('ø')[1:]
    except:
        False, []
    th_E.fin()
    return True, others



# **************************************************
#       POUR TESTS
# **************************************************
def sendMessage(host, port, message=''):
    # Établissement de la connexion :
    connexion = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        connexion.connect((host, port))
        message = message.encode('Utf8')
        connexion.send(message)
        connexion.close()
    except socket.error:
        print('CLIENT SOCKET ERROR (sendMessage)')
        return False
    return True



# **************************************************
#       PROGRAMME PRINCIPAL
# **************************************************
def main(host, port):
    # Programme principal - Établissement de la connexion :
    connexion = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        connexion.connect((host, port))
    except socket.error:
        print('CLIENT SOCKET ERROR (main)')
        return False
    print('CLIENT CONNECTED')

    # Dialogue avec le serveur : on lance deux threads pour gérer
    # indépendamment l'émission et la réception des messages :
    th_E = ThreadEmission(connexion)
    th_R = ThreadReception(connexion)
    th_E.start()
    th_R.start()
    return True


